/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/model/model.hpp>

#include <keel/asset.hpp>
#include <keel/module.hpp>

#include <nostalgia/core/palette.hpp>
#include <nostalgia/core/tilesheet.hpp>

#include "typeconv.hpp"

namespace nostalgia::core {

class CoreModule: public keel::Module {
	private:
		NostalgiaPaletteToPaletteConverter nostalgiaPaletteToPaletteConverter;
		TileSheetV1ToTileSheetConverter nostalgiaGraphicToTileSheetConverter;
		TileSheetToCompactTileSheetConverter tileSheetToCompactTileSheetConverter;
		TileSheetV2ToTileSheetConverter tileSheetV2ToTileSheetConverter;

	public:
		[[nodiscard]]
		ox::Vector<keel::TypeDescGenerator> types() const noexcept final {
			return {
				keel::generateTypeDesc<TileSheetV1>,
				keel::generateTypeDesc<TileSheetV2>,
				keel::generateTypeDesc<TileSheet>,
				keel::generateTypeDesc<CompactTileSheet>,
				keel::generateTypeDesc<NostalgiaPalette>,
				keel::generateTypeDesc<Palette>,
			};
		}

		[[nodiscard]]
		ox::Vector<const keel::BaseConverter*> converters() const noexcept final {
			return {
				&nostalgiaPaletteToPaletteConverter,
				&nostalgiaGraphicToTileSheetConverter,
				&tileSheetToCompactTileSheetConverter,
				&tileSheetV2ToTileSheetConverter,
			};
		}

		[[nodiscard]]
		ox::Vector<keel::PackTransform> packTransforms() const noexcept final {
			return {
				// convert tilesheets to CompactTileSheets
				[](keel::Context *ctx, ox::Buffer *buff) -> ox::Error {
					oxRequire(hdr, keel::readAssetHeader(*buff));
					const auto typeId = ox::buildTypeId(hdr.clawHdr.typeName, hdr.clawHdr.typeVersion, hdr.clawHdr.typeParams);
					if (typeId == ox::buildTypeId<TileSheetV1>() ||
						 typeId == ox::buildTypeId<TileSheetV2>() ||
						 typeId == ox::buildTypeId<TileSheet>()) {
						oxReturnError(keel::convertBuffToBuff<core::CompactTileSheet>(
									ctx, *buff, ox::ClawFormat::Metal).moveTo(buff));
					}
					return {};
				},
			};
		}
};

static CoreModule mod;
const keel::Module *keelModule() noexcept {
	return &mod;
}

}
